// Author:       Andreas F. Borchert
// Description:  RPN calculator

#include "Calculator.h"
#include <istream>
#include <sstream>

using namespace std;

Calculator::Calculator(FloatStack& stack) :
      opstack(stack)
{
} // constructor with stack parameter

float Calculator::calculate(const string& expr) throw(CalculatorException) {
   istringstream in(expr);
   string token;
   float result;
   try {
      while (in >> token) {
         if (token == "+" || token == "-" || token == "*" || token == "/") {
            float op2(opstack.top()); opstack.pop();
            float op1(opstack.top()); opstack.pop();
            float result;
            if (token == "+") {
               result = op1 + op2;
            } else if (token == "-") {
               result = op1 - op2;
            } else if (token == "*") {
               result = op1 * op2;
            } else {
               result = op1 / op2;
            }
            opstack.push(result);
         } else {
            istringstream floatin(token);
            float newop;
            if (floatin >> newop) {
               opstack.push(newop);
            } else {
               throw SyntaxError();
            }
         }
      }
      result = opstack.top(); opstack.pop();
      if (!opstack.empty()) {
         throw BadExpr();
      }
   } catch(FullStack) {
      throw StackFailure();
   } catch(EmptyStack) {
      throw BadExpr();
   }
   return result;
} // calculate
